@@ -56,10 +56,10 @@ typedef struct _snd_ctl_ops {
 
 
 struct _snd_ctl {
-	void *dl_handle;
+	void *open_func;
 	char *name;
 	snd_ctl_type_t type;
-	snd_ctl_ops_t *ops;
+	const snd_ctl_ops_t *ops;
 	void *private_data;
 	int nonblock;
 	int poll_fd;
@@ -89,9 +89,14 @@ struct _snd_hctl {
 };
 
 
+/* make local functions really local */
+#define snd_ctl_new	snd1_ctl_new
+
 int snd_ctl_new(snd_ctl_t **ctlp, snd_ctl_type_t type, const char *name);
 int _snd_ctl_poll_descriptor(snd_ctl_t *ctl);
 #define _snd_ctl_async_descriptor _snd_ctl_poll_descriptor
 int snd_ctl_hw_open(snd_ctl_t **handle, const char *name, int card, int mode);
 int snd_ctl_shm_open(snd_ctl_t **handlep, const char *name, const char *sockname, const char *sname, int mode);
 int snd_ctl_async(snd_ctl_t *ctl, int sig, pid_t pid);
+
+#define CTLINABORT(x) ((x)->nonblock == 2)
