@@ -274,8 +274,8 @@ static inline int snd_pcm_direct_semaphore_up(snd_pcm_direct_t *dmix, int sem_nu
 static inline int snd_pcm_direct_semaphore_final(snd_pcm_direct_t *dmix, int sem_num)
 {
 	if (dmix->locked[sem_num] != 1) {
-		assert(dmix->locked[sem_num] != 1);
-		abort();
+		SNDMSG("invalid semaphore count to finalize %d: %d", sem_num, dmix->locked[sem_num]);
+		return -EBUSY;
 	}
 	return snd_pcm_direct_semaphore_up(dmix, sem_num);
 }
