@@ -45,14 +45,14 @@ typedef struct snd_pcm_extplug_priv {
 	struct snd_ext_parm sparams[SND_PCM_EXTPLUG_HW_PARAMS];
 } extplug_priv_t;
 
-static int hw_params_type[SND_PCM_EXTPLUG_HW_PARAMS] = {
+static const int hw_params_type[SND_PCM_EXTPLUG_HW_PARAMS] = {
 	[SND_PCM_EXTPLUG_HW_FORMAT] = SND_PCM_HW_PARAM_FORMAT,
 	[SND_PCM_EXTPLUG_HW_CHANNELS] = SND_PCM_HW_PARAM_CHANNELS
 };
 
 #define is_mask_type(i) (hw_params_type[i] < SND_PCM_HW_PARAM_FIRST_INTERVAL)
 
-static unsigned int excl_parbits[SND_PCM_EXTPLUG_HW_PARAMS] = {
+static const unsigned int excl_parbits[SND_PCM_EXTPLUG_HW_PARAMS] = {
 	[SND_PCM_EXTPLUG_HW_FORMAT] = (SND_PCM_HW_PARBIT_FORMAT|
 				       SND_PCM_HW_PARBIT_SUBFORMAT |
 				       SND_PCM_HW_PARBIT_SAMPLE_BITS),
@@ -425,7 +425,37 @@ static int snd_pcm_extplug_close(snd_pcm_t *pcm)
 	return 0;
 }
 
-static snd_pcm_ops_t snd_pcm_extplug_ops = {
+static snd_pcm_chmap_query_t **snd_pcm_extplug_query_chmaps(snd_pcm_t *pcm)
+{
+	extplug_priv_t *ext = pcm->private_data;
+
+	if (ext->data->version >= 0x010002 &&
+	    ext->data->callback->query_chmaps)
+		return ext->data->callback->query_chmaps(ext->data);
+	return snd_pcm_generic_query_chmaps(pcm);
+}
+
+static snd_pcm_chmap_t *snd_pcm_extplug_get_chmap(snd_pcm_t *pcm)
+{
+	extplug_priv_t *ext = pcm->private_data;
+
+	if (ext->data->version >= 0x010002 &&
+	    ext->data->callback->get_chmap)
+		return ext->data->callback->get_chmap(ext->data);
+	return snd_pcm_generic_get_chmap(pcm);
+}
+
+static int snd_pcm_extplug_set_chmap(snd_pcm_t *pcm, const snd_pcm_chmap_t *map)
+{
+	extplug_priv_t *ext = pcm->private_data;
+
+	if (ext->data->version >= 0x010002 &&
+	    ext->data->callback->set_chmap)
+		return ext->data->callback->set_chmap(ext->data, map);
+	return snd_pcm_generic_set_chmap(pcm, map);
+}
+
+static const snd_pcm_ops_t snd_pcm_extplug_ops = {
 	.close = snd_pcm_extplug_close,
 	.info = snd_pcm_generic_info,
 	.hw_refine = snd_pcm_extplug_hw_refine,
@@ -438,6 +468,9 @@ static snd_pcm_ops_t snd_pcm_extplug_ops = {
 	.async = snd_pcm_generic_async,
 	.mmap = snd_pcm_generic_mmap,
 	.munmap = snd_pcm_generic_munmap,
+	.query_chmaps = snd_pcm_extplug_query_chmaps,
+	.get_chmap = snd_pcm_extplug_get_chmap,
+	.set_chmap = snd_pcm_extplug_set_chmap,
 };
 
 #endif /* !DOC_HIDDEN */
@@ -641,8 +674,11 @@ int snd_pcm_extplug_create(snd_pcm_extplug_t *extplug, const char *name,
 	assert(extplug->callback->transfer);
 	assert(slave_conf);
 
-	if (extplug->version != SND_PCM_EXTPLUG_VERSION) {
-		SNDERR("extplug: Plugin version mismatch\n");
+	/* We support 1.0.0 to current */
+	if (extplug->version < 0x010000 ||
+	    extplug->version > SND_PCM_EXTPLUG_VERSION) {
+		SNDERR("extplug: Plugin version mismatch: 0x%x\n",
+		       extplug->version);
 		return -ENXIO;
 	}
 
@@ -668,7 +704,7 @@ int snd_pcm_extplug_create(snd_pcm_extplug_t *extplug, const char *name,
 	ext->plug.undo_write = snd_pcm_plugin_undo_write_generic;
 	ext->plug.gen.slave = spcm;
 	ext->plug.gen.close_slave = 1;
-	if (extplug->callback->init)
+	if (extplug->version >= 0x010001 && extplug->callback->init)
 		ext->plug.init = snd_pcm_extplug_init;
 
 	err = snd_pcm_new(&pcm, SND_PCM_TYPE_EXTPLUG, name, stream, mode);
