@@ -44,8 +44,6 @@ typedef struct {
 	snd_pcm_slave_xfer_areas_func_t write;
 	snd_pcm_slave_xfer_areas_undo_func_t undo_read;
 	snd_pcm_slave_xfer_areas_undo_func_t undo_write;
-	snd_pcm_sframes_t (*client_frames)(snd_pcm_t *pcm, snd_pcm_sframes_t frames);
-	snd_pcm_sframes_t (*slave_frames)(snd_pcm_t *pcm, snd_pcm_sframes_t frames);
 	int (*init)(snd_pcm_t *pcm);
 	snd_pcm_uframes_t appl_ptr, hw_ptr;
 	snd_atomic_write_t watom;
@@ -60,10 +58,16 @@ typedef struct {
 	snd1_pcm_plugin_undo_read_generic
 #define snd_pcm_plugin_undo_write_generic \
 	snd1_pcm_plugin_undo_write_generic
+#define snd_pcm_plugin_rewind \
+	snd1_pcm_plugin_rewind
+#define snd_pcm_plugin_forward \
+	snd1_pcm_plugin_forward
 
 void snd_pcm_plugin_init(snd_pcm_plugin_t *plugin);
+snd_pcm_sframes_t snd_pcm_plugin_rewind(snd_pcm_t *pcm, snd_pcm_uframes_t frames);
+snd_pcm_sframes_t snd_pcm_plugin_forward(snd_pcm_t *pcm, snd_pcm_uframes_t frames);
 
-extern snd_pcm_fast_ops_t snd_pcm_plugin_fast_ops;
+extern const snd_pcm_fast_ops_t snd_pcm_plugin_fast_ops;
 
 snd_pcm_sframes_t snd_pcm_plugin_undo_read_generic
      (snd_pcm_t *pcm,
