@@ -136,18 +136,15 @@ static snd_pcm_uframes_t snd_pcm_share_slave_avail(snd_pcm_share_slave_t *slave)
 static snd_pcm_uframes_t _snd_pcm_share_slave_forward(snd_pcm_share_slave_t *slave)
 {
 	struct list_head *i;
-	snd_pcm_uframes_t buffer_size, boundary;
-	snd_pcm_uframes_t slave_appl_ptr;
+	snd_pcm_uframes_t buffer_size;
 	snd_pcm_sframes_t frames, safety_frames;
 	snd_pcm_sframes_t min_frames, max_frames;
 	snd_pcm_uframes_t avail, slave_avail;
 	snd_pcm_uframes_t slave_hw_avail;
 	slave_avail = snd_pcm_share_slave_avail(slave);
-	boundary = slave->pcm->boundary;
 	buffer_size = slave->pcm->buffer_size;
 	min_frames = slave_avail;
 	max_frames = 0;
-	slave_appl_ptr = *slave->pcm->appl.ptr;
 	list_for_each(i, &slave->clients) {
 		snd_pcm_share_t *share = list_entry(i, snd_pcm_share_t, list);
 		snd_pcm_t *pcm = share->pcm;
@@ -598,7 +595,7 @@ static int snd_pcm_share_hw_refine_slave(snd_pcm_t *pcm, snd_pcm_hw_params_t *pa
 static int snd_pcm_share_hw_params_slave(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 {
 	snd_pcm_share_t *share = pcm->private_data;
-	return _snd_pcm_hw_params(share->slave->pcm, params);
+	return _snd_pcm_hw_params_internal(share->slave->pcm, params);
 }
 
 static int snd_pcm_share_hw_refine(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
@@ -1308,7 +1305,7 @@ static void snd_pcm_share_dump(snd_pcm_t *pcm, snd_output_t *out)
 	snd_pcm_dump(slave->pcm, out);
 }
 
-static snd_pcm_ops_t snd_pcm_share_ops = {
+static const snd_pcm_ops_t snd_pcm_share_ops = {
 	.close = snd_pcm_share_close,
 	.info = snd_pcm_share_info,
 	.hw_refine = snd_pcm_share_hw_refine,
@@ -1323,7 +1320,7 @@ static snd_pcm_ops_t snd_pcm_share_ops = {
 	.munmap = snd_pcm_share_munmap,
 };
 
-static snd_pcm_fast_ops_t snd_pcm_share_fast_ops = {
+static const snd_pcm_fast_ops_t snd_pcm_share_fast_ops = {
 	.status = snd_pcm_share_status,
 	.state = snd_pcm_share_state,
 	.hwsync = snd_pcm_share_hwsync,
@@ -1529,7 +1526,7 @@ int snd_pcm_share_open(snd_pcm_t **pcmp, const char *name, const char *sname,
 	pcm->private_data = share;
 	pcm->poll_fd = share->client_socket;
 	pcm->poll_events = stream == SND_PCM_STREAM_PLAYBACK ? POLLOUT : POLLIN;
-	pcm->monotonic = pcm->monotonic;
+	pcm->monotonic = slave->pcm->monotonic;
 	snd_pcm_set_hw_ptr(pcm, &share->hw_ptr, -1, 0);
 	snd_pcm_set_appl_ptr(pcm, &share->appl_ptr, -1, 0);
 
