@@ -63,7 +63,7 @@ contents of written buffer - passed by snd_rawmidi_write() - atomically
 to output ring buffer in the kernel space. This flag also means that device
 is not opened exclusively, so more applications can share given rawmidi device.
 Note that applications must send the whole MIDI message including the running status,
-because another writting application might break the MIDI message in the output
+because another writing application might break the MIDI message in the output
 buffer.
 
 \subsection rawmidi_open_sync Sync open (flag)
@@ -153,7 +153,7 @@ static int snd_rawmidi_params_default(snd_rawmidi_t *rawmidi, snd_rawmidi_params
 	assert(params);
 	params->buffer_size = page_size();
 	params->avail_min = 1;
-	params->no_active_sensing = 0;
+	params->no_active_sensing = 1;
 	return 0;
 }
 
@@ -201,6 +201,7 @@ static int snd_rawmidi_open_conf(snd_rawmidi_t **inputp, snd_rawmidi_t **outputp
 	if (err >= 0) {
 		if (snd_config_get_type(type_conf) != SND_CONFIG_TYPE_COMPOUND) {
 			SNDERR("Invalid type for RAWMIDI type %s definition", str);
+			err = -EINVAL;
 			goto _err;
 		}
 		snd_config_for_each(i, next, type_conf) {
@@ -987,9 +988,10 @@ ssize_t snd_rawmidi_read(snd_rawmidi_t *rawmidi, void *buffer, size_t size)
 	return (rawmidi->ops->read)(rawmidi, buffer, size);
 }
 
+#ifndef DOC_HIDDEN
 int snd_rawmidi_conf_generic_id(const char *id)
 {
-	static const char *ids[] = {
+	static const char ids[][8] = {
 		"comment",
 		"type",
 		"hint",
@@ -1002,3 +1004,4 @@ int snd_rawmidi_conf_generic_id(const char *id)
 	}
 	return 0;
 }
+#endif
