@@ -23,7 +23,6 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#define __USE_GNU
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include "timer_local.h"
@@ -36,7 +35,7 @@ const char *_snd_module_timer_hw = "";
 #define SNDRV_FILE_TIMER		ALSA_DEVICE_DIRECTORY "timer"
 #define SNDRV_TIMER_VERSION_MAX	SNDRV_PROTOCOL_VERSION(2, 0, 5)
 
-#define SNDRV_TIMER_IOCTL_STATUS_OLD	_IOW('T', 0x14, struct sndrv_timer_status)
+#define SNDRV_TIMER_IOCTL_STATUS_OLD	_IOW('T', 0x14, struct snd_timer_status)
 
 enum {
 	SNDRV_TIMER_IOCTL_START_OLD = _IO('T', 0x20),
@@ -209,7 +208,7 @@ static ssize_t snd_timer_hw_read(snd_timer_t *handle, void *buffer, size_t size)
 	return result;
 }
 
-static snd_timer_ops_t snd_timer_hw_ops = {
+static const snd_timer_ops_t snd_timer_hw_ops = {
 	.close = snd_timer_hw_close,
 	.nonblock = snd_timer_hw_nonblock,
 	.async = snd_timer_hw_async,
@@ -226,7 +225,7 @@ int snd_timer_hw_open(snd_timer_t **handle, const char *name, int dev_class, int
 {
 	int fd, ver, tmode, ret;
 	snd_timer_t *tmr;
-	struct sndrv_timer_select sel;
+	struct snd_timer_select sel;
 
 	*handle = NULL;
 
@@ -236,17 +235,6 @@ int snd_timer_hw_open(snd_timer_t **handle, const char *name, int dev_class, int
 	fd = snd_open_device(SNDRV_FILE_TIMER, tmode);
 	if (fd < 0)
 		return -errno;
-#if 0
-	/*
-	 * this is bogus, an application have to care about open filedescriptors
-	 */
-	if (fcntl(fd, F_SETFD, FD_CLOEXEC) != 0) {
-		SYSERR("fcntl FD_CLOEXEC failed");
-		ret = -errno;
-		close(fd);
-		return ret;
-	}
-#endif
 	if (ioctl(fd, SNDRV_TIMER_IOCTL_PVERSION, &ver) < 0) {
 		ret = -errno;
 		close(fd);
